#!/bin/bash
set -x
fail() { [ $# -eq 0 ] || echo "$@"; exit 1; }

find . -iname "*build_properties" -exec cp {} . \; ||
    echo "Unable to copy build properties, this might be v2"

[ -z "${SERIAL}" -a -z "${SUITE}" -a -e "build_properties" ] && {
    source build_properties ||
    fail "Failed to read build_properties. I don't know what I'm doing!";
}

STREAM="${STREAM:-daily}"
# Read in the common functions
my_dir="$( cd "$( dirname "$0" )" && pwd )"
base_dir=$(dirname ${my_dir})
export PATH="${base_dir}:${my_dir}:${PATH}"
source "${base_dir}/functions/locker"
source "${base_dir}/functions/common"
source "${base_dir}/functions/retry"
source ${my_dir}/build_lib.sh
select_build_config

export WORKSPACE="${WORKSPACE:-$WORKSPACE_R}"
out_f="${WORKSPACE}/maas-${SUITE}-${STREAM}-config.sh"
raw_f="${WORKSPACE}/${SUITE}-output.raw"
query_t="${WORKSPACE}/cloud-images-query.tar"
base_name="${SUITE}-server-cloudimg"
rel_base_name="ubuntu-${VERSION:-$(ubuntu-adj2version ${SUITE})}-${stream//-/}-server-cloudimg"

export maas_branch_v1="http://bazaar.launchpad.net/~smoser/maas/maas.ubuntu.com.images-ephemeral"
export maas_branch="${MAAS_BRANCH:-$maas_branch_v1}"

case "${STREAM}" in
    release)  build_f="/srv/ec2-images/releases/${SUITE}/release-${SERIAL}";
              base_name=${rel_base_name};
              out_d="/srv/maas-images/ephemeral/releases/${SUITE}/release-${SERIAL}"
              ;;
    daily)    build_f="/srv/ec2-images/${SUITE}/${SERIAL}";
              out_d="/srv/maas-images/ephemeral/daily/${SUITE}/${SERIAL}";
              ;;
    alpha*|beta*)  build_f="/srv/ec2-images/releases/${SUITE}/${STREAM}";
              base_name=${rel_base_name};
              out_d="/srv/maas-images/ephemeral/releases/${SUITE}/${STREAM}";
              ;;
    *)        fail "Unknown stream ${STREAM}.";;
esac

final_out_d="${out_d}"

[ -e "${final_out_d}" -a "${REBUILD:-false}" = "false" ] &&
    fail "Build already exists. Rebuild is set to false. Failing this build"

if [ ! -e "${query_t}" ]; then

    if  [ "${MAASv2:-0}" -eq 1 ]; then
        # MAAS v2 doesn't need this information
        out_d="/tmp/maas_final"
        touch ${WORKSPACE}/maasv2
        tar cvf ${query_t} ${WORKSPACE}/maasv2

        if  [ -e "${WORKSPACE}/tmp/maas-final" ]; then
            tar cvf ${query_t} maas-final ||
                fail "Failed to create tarball of MAAS images"
        fi

    else
        # MAAS v1 need information
        tar cvf ${query_t} \
            ${QUERY_D:-/srv/ec2-images/query} \
            ${build_f}  \
            --exclude "*img" --exclude "*azure*" --exclude "*html" \
            --exclude "*armel*" --exclude "*root.tar.gz" \
            --exclude "*floppy" ||
                fail "Failed to pack up build elements for MAAS builder";
    fi
fi

# Select the right template
tmpl="${kvm_builder}/templates/img-maas.tmpl"
[ "${MAASv2:-0}" -eq 1 ] && tmpl="${tmpl//maas.tmpl/maasv2.tmpl}"

# Construct the right template
maas_config.sh \
    --distro "${SUITE}" \
    --stream "${STREAM}" \
    --template "${tmpl}" \
    --base-name "${base_name}" \
    --local "${build_f}" \
    --serial "${SERIAL}" \
    --out "${out_f}" \
    --maas-branch "${maas_branch}" \
    --out_d "${out_d}" ||
    fail "Failed to configure KVM instance for building"
set +x

ci_cfg="${kvm_builder}/config/cloud-maasv2.cfg"
[ "$(uname -m)" == "ppc64" ] && ci_cfg="${kvm_builder}/config/cloud-trusty-pp64el.cfg"

# Launch KVM to do the work
launch_kvm.sh \
    --id ${BUILD_ID} \
    --user-data "${out_f}" \
    --cloud-config "${ci_cfg}" \
    --extra-disk "${query_t}" \
    --disk-gb 50 \
    --raw-disk "${raw_f}" \
    --raw-size 20 \
    --mem 1G \
    --img-url ${BUILDER_CLOUD_IMAGE} ||
        fail "KVM instance failed to build image."

# Extract the result set
tar -xvvf "${raw_f}" ||
    fail "Failed to extract information from instance"

# Useful for off-host builds, like ppc64el. Just make sure that any-off host
# builds are done before the on-hosts builds.

[ "${BUILD_ONLY:-0}" -eq 1 ] && exit 0
[ "${MAASv2:-0}" -eq 1 ] && exit 0

# Extracted reslts should be here
[ ! -e "${WORKSPACE}/${out_d}" ] && fail "Expected result directory is missing: ${WORKSPACE}/${out_d}"

# Checksum the results (and sign 'em)
export CDIMAGE_ROOT="/srv/builder/vmbuilder/cdimage"
/srv/builder/vmbuilder/bin/cronrun checksum-directory "${WORKSPACE}/${out_d}" ||
    fail "Failed to create checksums and GPG signatures"

set -x
# Put the bits where they go...
mkdir -p "${final_out_d}" &&
    cp -a ${WORKSPACE}${out_d}/* "${final_out_d}" &&
    echo "Copied bits to final location ${final_out_d}" ||
    fail "Unable to copy build bits to final location"

# Produce build-info
cat << EOF > "${final_out_d}/build-info.txt"
serial=${SERIAL}
orig_prefix=${SUITE}-ephemeral-maas
suite=${SUITE}
build_name=ephemeral
EOF

# Clean up the dailies
if [ "${STREAM}" = "daily" ]; then
    base_d="${out_d%/*}"
    builds=( $(find ${base_d} -maxdepth 1 -mindepth 1 -type d | sort -r) )
    build_count=${#builds[@]}

    # Delete all but the
    if [ ${build_count} -gt 6 ]; then
        for item in $(seq 6 ${build_count})
        do
            [ -e "${builds[$item]}" ] && {
                rm -rf ${builds[$item]};
                echo "Build ${SUITE} ${builds[$item]##*/} has been deleted";
            }
        done

        for item in $(seq 0 5)
        do
                echo "Preserving ${SUITE} ${builds[$item]##*/}"
        done
    else
        echo "No builds marked for deletion"
    fi
fi

# Generate the Query2 tree
src_tree="${WORKSPACE}/maas_src"
bzr branch "${maas_branch_v1}" "${src_tree}"
${src_tree}/tree2query \
    --commit-msg "Build ${BUILD_ID}" \
    --namespace maas \
    /srv/maas-images

# Update current
if [ "${STREAM}" = "daily" ]; then
    cur_d="/srv/maas-images/ephemeral/daily/${SUITE}/current"
    [ -e "${cur_d}" ] && rm "${cur_d}"
    ln -s "${final_out_d}" "${cur_d}" ||
        echo "Failed to update ${cur_d}"
fi


# Remove the results
rm "${raw_f}" ||
    fail "Failed to clean up files!"

