#!/usr/bin/python

# The legacy Jenkins test is buggy due to the cloud being the cloud.
# This is a test evaluator that determines whether or not to pass/fail
# a jenkins build flow.

# Valid ball colors:
# blue: Pass
# grey: waiting
# grey_anime: executing
# red: Fail
# yellow: (also) Fail - treating UNSTABLE as the test failure it is

import argparse
import json
import requests
import sys
import time
import urlparse

parser = argparse.ArgumentParser()
parser.add_argument('--host',
        action="store",
        default="localhost:8080",
        help="Host address to hit")
parser.add_argument('--test',
        action="store",
        default=None,
        help="Test suite to parse")
parser.add_argument('--okay',
        action="store",
        type=int,
        default=10,
        help="How many failures are okay to pass test")
parser.add_argument('--region_okay',
        action="store",
        type=int,
        default=3,
        help="How many failure per_region are okay to pass")
parser.add_argument('--test_okay',
        action="store",
        type=int,
        default=3,
        help="Number of failures within a test type are okay to pass")
parser.add_argument('--test_region_okay',
        action="store",
        type=int,
        default=3,
        help="Okay to pass X failures in region per test")
parser.add_argument('--suite',
        action="store",
        default=None,
        help="Search runs for a particular suite")
parser.add_argument('--milestone-label',
        action="store",
        default=None,
        help="Search runs for a particular milestone label")
parser.add_argument('--serial',
        action="store",
        default=None,
        help="Search runs for a particular serial")

opts = parser.parse_args()

sys.stderr.write("Using host: %s\n" % opts.host)

if not opts.test:
    sys.stderr.write("Must define test with --test\n")
    sys.exit(3)

if not opts.suite and (opts.milestone_label or opts.serial):
    sys.stderr.write(
        '--milestone-label and --serial can only be specified with --suite\n')
    sys.exit(3)

counts={}

def _convert_public_jenkins_url(url):
    relative_url = urlparse.urlparse(url).path
    return 'http://{}{}'.format(opts.host, relative_url)


def _get_parameter_dict(parameter_list):
    parameters = {}
    for parameter_item in parameter_list:
        parameters[parameter_item['name']] = parameter_item['value']
    return parameters


def _none_or_match(option, needle):
    return option is None or option == needle


def get_build(json_d):
    if opts.suite is None:
        return json_d['builds'][0]
    for part_build in json_d['builds']:
        sys.stderr.write('Considering build {}... '.format(
            part_build['number']))
        sys.stderr.flush()
        build = requests.get(_convert_public_jenkins_url(
            '{}/api/json'.format(part_build['url']))).json()
        for action in build['actions']:
            if 'parameters' not in action:
                continue
            parameters = _get_parameter_dict(action['parameters'])
            if (parameters['SUITE'] == opts.suite
                    and _none_or_match(opts.milestone_label,
                                       parameters['MILESTONE_LABEL'])
                    and _none_or_match(opts.serial, parameters['SERIAL'])):
                sys.stderr.write('parameters match\n')
                return build
            sys.stderr.write('parameters do not match\n')
            break
    raise Exception('No build matching parameters')

def get_build_results(json_d):
    build_url = get_build(json_d)['url']
    build_json = requests.get('{}/api/json?depth=1'.format(
        _convert_public_jenkins_url(build_url))).json()
    fake_configurations = []
    for run in build_json['runs']:
        if run['number'] != build_json['number']:
            continue
        if run['result'] == 'SUCCESS':
            color = 'blue'
        # Failed builds indicate a Jenkins job failure
        elif run['result'] == 'FAILURE':
            color = 'red'
        # Unstable builds are the result of a test failure
        elif run['result'] == 'UNSTABLE':
            color = 'red'
        elif run['result'] == 'ABORTED' or run['result'] is None:
            color = 'grey'
        else:
            assert False, run['result']
        fake_configurations.append({
            'name': run['url'].split('/')[5],
            'color': color,
        })
    return fake_configurations

def check():
    colors = {}
    url = "http://%s/job/%s/api/json" % (opts.host, opts.test)
    response = requests.get(url)
    json_d = response.json()
    results = get_build_results(json_d)
    c = {}

    for result in results:
        color = result['color']

        # We get something like:
        # ARCH=amd64,REGION=eu-west-1,STORAGE=ebs,TEST=simple-user-data,VIRT_TYPE=hvm,label=EC2
        test_attributes = result['name'].split(',')
        (arch, test, region) = (None, None, None)
        for attribute, value in [
                x.split('=') for x in test_attributes if  '=' in x ]:
            if attribute == 'REGION':
                region = value
            if attribute == "TEST":
                test = value
            if attribute == "ARCH":
                arch = value

            # only get what we need
            if test and region and arch:
                break

        # Don't fail tests for 32-bit regions
        if 'i386' in arch and region in ('eu-central-1', 'sa-east-1'):
            print("Skipping 32-bit testing for {}: {}".format(region, test))
            continue

        if region not in c:
            c[region] = {'colors': {},
                         'tests': {}}

        if color not in c[region]['colors']:
            c[region]['colors'][color] = 1
        else:
            c[region]['colors'][color] += 1

        if test not in c[region]['tests']:
            c[region]['tests'][test] = {color: 1}
        elif color not in c[region]['tests'][test]:
            c[region]['tests'][test] = {color: 1}
        else:
            c[region]['tests'][test][color] += 1

    max_failures = 0
    total_failures = 0
    total_success = 0
    still_testing = 0
    test_type = {}
    max_regional_failures = 0

    for region in c:
        reg = c.get(region)
        colors = reg.get('colors')
        total_success += colors.get('blue', 0)
        still_testing += colors.get('gray_anime', 0)
        fail_count = colors.get('red', 0)
        total_failures += fail_count
        if fail_count > max_failures:
            max_failures = fail_count
        if fail_count > max_regional_failures:
            max_regional_failures = fail_count

        tests = reg.get('tests')
        for test in tests:
            if test not in test_type:
                test_type[test] = {}
            results = tests.get(test)

            for color in results:
                color_count = results.get(color, 0)
                if color not in test_type[test]:
                    test_type[test][color] = color_count
                else:
                    test_type[test][color] += color_count

    test_red_count = 0
    max_red_test_region = 0
    for test in test_type:
        results = test_type.get(test)
        red = results.get('red', 0)
        test_red_count += red
        if red > max_red_test_region:
            max_red_test_region = red

    dump = json.dumps(c, indent=3, sort_keys=True)
    print("\nResults from build are:\n%s\n" % dump)

    c.update({'test_red_count': test_red_count,
              'max_red_test_region': max_red_test_region,
              'test_types': test_type,
              'max_regional_failures': max_regional_failures,
              'max_failures': max_failures,
              'total_failures': total_failures,
              'success': total_success})
    return c


counts = check()

# Wait for test to complete
while 'grey' in counts:
    print("Sleeping 5 minutes to wait for build completion")
    time.sleep(300)
    counts = check()

# Get our values out for parsing
max_failures = counts.get('max_failures', 0)
total_failures = counts.get('total_failures', 0)
max_regional_failures = counts.get('max_regional_failures', 0)
test_red_count = counts.get('test_red_count', 0)
max_red_test_region = counts.get('max_red_test_region', 0)

# Get some pretty output
print("Total successful tests:           [ %s ] " % counts['success'])
print("Regional max failures:            [ %s/%s ]" % (
        max_regional_failures, opts.region_okay))
print("Regional max per test failures:   [ %s/%s ]" % (
        max_red_test_region, opts.test_region_okay))
print("Max per test failures:            [ %s/%s ]" % (
        test_red_count, opts.test_okay))
print("Total failures:                   [ %s/%s ]" % (
        max_failures, opts.okay))

# Now gate-check the failures
if total_failures > opts.okay or max_regional_failures > opts.region_okay:
    print("Regional failures or total failures exceed thresholds. Failing.")
    sys.exit(2)
elif test_red_count > opts.test_okay:
    print("Max fail count per test exceeded")
    sys.exit(3)
elif max_red_test_region > opts.test_region_okay:
    print("Max fail count per test per region exceeed")
    sys.exit(4)
else:
    print("Failure rate is below fail threshold. Passing.")
    sys.exit(0)

